import { HostListener, Directive, ElementRef, Input, Output, EventEmitter, Renderer } from '@angular/core';
var ClipboardDirective = (function () {
    /**
     * @param {?} elmRef
     * @param {?} renderer
     */
    function ClipboardDirective(elmRef, renderer) {
        this.elmRef = elmRef;
        this.renderer = renderer;
        this.cbOnSuccess = new EventEmitter();
        this.cbOnError = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ClipboardDirective.prototype.ngOnInit = function () {
        // let option: Clipboard.Options;
        // option = !!this.targetElm ? { target: () => <any>this.targetElm } : { text: () => this.cbContent };
        // this.clipboard = new Clipboard(this.elmRef.nativeElement, option);
        // this.clipboard.on('success', (e) => this.cbOnSuccess.emit(e));
        // this.clipboard.on('error', (e) => this.cbOnError.emit(e));
    };
    /**
     * @return {?}
     */
    ClipboardDirective.prototype.ngOnDestroy = function () {
        // if (this.clipboard) {
        //     this.clipboard.destroy();
        // }
    };
    /**
     * @param {?} button
     * @return {?}
     */
    ClipboardDirective.prototype.onClick = function (button) {
        if (this.targetElm) {
            this.selectedText = this.targetElm.nativeElement.value;
            this.selectTarget(this.targetElm.nativeElement);
            this.handleResult(this.copyText());
            this.clearSelection(this.targetElm.nativeElement, window);
        }
    };
    /**
     * @param {?} nativeElement
     * @return {?}
     */
    ClipboardDirective.prototype.selectTarget = function (nativeElement) {
        // TODO: handle contenteditable or selection
        if (nativeElement.nodeName === 'INPUT' || nativeElement.nodeName === 'TEXTAREA') {
            // TODO: might need to set readonly first
            this.renderer.invokeElementMethod(nativeElement, 'select');
            this.renderer.invokeElementMethod(nativeElement, 'setSelectionRange', [0, nativeElement.value.length]);
        }
    };
    /**
     * @return {?}
     */
    ClipboardDirective.prototype.copyText = function () {
        try {
            // TODO: shouldn't use document directly
            return document.execCommand('copy');
        }
        catch (err) {
            throw new Error('fail to perform copy');
        }
    };
    /**
     * Fires an event based on the copy operation result.
    \@param {Boolean} succeeded
     * @param {?} succeeded
     * @return {?}
     */
    ClipboardDirective.prototype.handleResult = function (succeeded) {
        if (succeeded) {
            this.cbOnSuccess.emit({ isSuccess: true, content: this.selectedText });
        }
        else {
            this.cbOnError.emit({ isSuccess: false });
        }
    };
    /**
     * Removes current selection and focus from `target` element.
     * @param {?} nativeElement
     * @param {?} window
     * @return {?}
     */
    ClipboardDirective.prototype.clearSelection = function (nativeElement, window) {
        nativeElement && nativeElement.blur();
        // TODO: should inject window
        window.getSelection().removeAllRanges();
    };
    return ClipboardDirective;
}());
export { ClipboardDirective };
ClipboardDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxClipboard]'
            },] },
];
/**
 * @nocollapse
 */
ClipboardDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Renderer, },
]; };
ClipboardDirective.propDecorators = {
    'targetElm': [{ type: Input, args: ['ngxClipboard',] },],
    'cbContent': [{ type: Input },],
    'cbOnSuccess': [{ type: Output },],
    'cbOnError': [{ type: Output },],
    'onClick': [{ type: HostListener, args: ['click', ['$event.target'],] },],
};
function ClipboardDirective_tsickle_Closure_declarations() {
    /** @type {?} */
    ClipboardDirective.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    ClipboardDirective.ctorParameters;
    /** @type {?} */
    ClipboardDirective.propDecorators;
    /** @type {?} */
    ClipboardDirective.prototype.targetElm;
    /** @type {?} */
    ClipboardDirective.prototype.cbContent;
    /** @type {?} */
    ClipboardDirective.prototype.cbOnSuccess;
    /** @type {?} */
    ClipboardDirective.prototype.cbOnError;
    /** @type {?} */
    ClipboardDirective.prototype.selectedText;
    /** @type {?} */
    ClipboardDirective.prototype.elmRef;
    /** @type {?} */
    ClipboardDirective.prototype.renderer;
}
//# sourceMappingURL=clipboard.directive.js.map