import { HostListener, Directive, ElementRef, Input, Output, EventEmitter, OnInit, OnDestroy, Renderer } from '@angular/core';
@Directive({
    selector: '[ngxClipboard]'
})
export class ClipboardDirective implements OnInit, OnDestroy {
    // public clipboard: Clipboard;

    @Input('ngxClipboard') public targetElm: ElementRef;

    @Input() public cbContent: string;

    @Output() public cbOnSuccess: EventEmitter<any> = new EventEmitter<any>();

    @Output() public cbOnError: EventEmitter<any> = new EventEmitter<any>();
    private selectedText: string;
    constructor(
        private elmRef: ElementRef,
        private renderer: Renderer
    ) { }

    public ngOnInit() {
        // let option: Clipboard.Options;
        // option = !!this.targetElm ? { target: () => <any>this.targetElm } : { text: () => this.cbContent };
        // this.clipboard = new Clipboard(this.elmRef.nativeElement, option);
        // this.clipboard.on('success', (e) => this.cbOnSuccess.emit(e));
        // this.clipboard.on('error', (e) => this.cbOnError.emit(e));
    }

    public ngOnDestroy() {
        // if (this.clipboard) {
        //     this.clipboard.destroy();
        // }
    }

    @HostListener('click', ['$event.target']) private onClick(button) {
        if (this.targetElm) {
            this.selectedText = this.targetElm.nativeElement.value;
            this.selectTarget(this.targetElm.nativeElement);
            this.handleResult(this.copyText());
            this.clearSelection(this.targetElm.nativeElement, window);
        }
    }


    // select the native element
    private selectTarget(nativeElement: any) {
        // TODO: handle contenteditable or selection
        if (nativeElement.nodeName === 'INPUT' || nativeElement.nodeName === 'TEXTAREA') {
            // TODO: might need to set readonly first
            this.renderer.invokeElementMethod(nativeElement, 'select');
            this.renderer.invokeElementMethod(nativeElement, 'setSelectionRange', [0, nativeElement.value.length]);
        }
    }

    private copyText(): boolean {
        try {
            // TODO: shouldn't use document directly
            return document.execCommand('copy');
        } catch (err) {
            throw new Error('fail to perform copy');
        }
    }

    /**
     * Fires an event based on the copy operation result.
     * @param {Boolean} succeeded
     */
    private handleResult(succeeded) {
        if (succeeded) {
            this.cbOnSuccess.emit({ isSuccess: true, content: this.selectedText });
        } else {
            this.cbOnError.emit({ isSuccess: false });
        }
    }
    /**
       * Removes current selection and focus from `target` element.
       */
    private clearSelection(nativeElement: any, window: Window) {
        nativeElement && nativeElement.blur();
        // TODO: should inject window
        window.getSelection().removeAllRanges();
    }
}
